from AccessControl import ClassSecurityInfo, AuthEncoding

from Products.PluggableAuthService.utils import createViewName, classImplements, implementedBy

from Products.Membrane.interfaces.group import IGroup

from Products.Membrane.config import PROJECTNAME, TOOLNAME
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseSchema, Schema, BaseFolder
from Products.Archetypes.public import ReferenceField, ReferenceWidget, StringField, StringWidget
from Products.Archetypes.public import registerType
from Products.Archetypes.public import DisplayList


SimpleSchema = BaseSchema + Schema((
    StringField('groupName',
                languageIndependent = 1,
                widget = StringWidget(description = "Group name.")
               ),
    ReferenceField(
        name='manager',
        relationship='managesProject',
        allowed_types=('Member','SimpleMember',),
        vocabulary='listUsers',
        languageIndependent = 1,
        widget=ReferenceWidget(label='Group manager',
                               description='The manager of this project.'),
                ),
    ReferenceField(
        name="members",
        relationship='participatesInProject',
        multiValued=1,
        languageIndependent = 1,
        allowed_types=('Member','SimpleMember',),
        vocabulary='listUsers',
        widget=ReferenceWidget( label='Associated group members',
                                description='Members of this group that are really from other groups',
                                ),
                ),
    ))


class SimpleGroup(BaseFolder):
    """A simple member archetype"""
    schema = SimpleSchema
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    def getUserId(self):
        return self.getGroupName()

    def getGroupMembers(self):
        # All references and all subobjects that are members
        members = dict.fromkeys([m.getUserId() for m in self.getRefs('participatesInProject')])
        mt = getToolByName(self, TOOLNAME)
        for m in mt.unrestrictedSearchResults(object_implements='IUser', path='/'.join(self.getPhysicalPath())):
            members[m.getUserId] = 1
        return tuple(members.keys())

    def listUsers(self):
        """
        Return a DisplayList of users
        """
        catalog = getToolByName(self, TOOLNAME)

        results = catalog(object_implements='IUser')
        value = []
        for r in results:
            key = r.getUserName is not None and r.getUserName.strip() or r.getUserId
            value.append((key.lower(), (r.UID, key)))
        value.sort()
        value = [r for throwaway, r in value]
        value.insert(0, ('', '<no reference>'))
        return DisplayList(value)


classImplements( SimpleGroup
               , *tuple(implementedBy(BaseFolder)) +
                 (IGroup,))


registerType(SimpleGroup, PROJECTNAME)
