from AccessControl import ClassSecurityInfo, AuthEncoding
from Acquisition import aq_chain, aq_inner

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.utils import createViewName, classImplements, implementedBy

from Products.Membrane.interfaces.group import IGroup
from Products.Membrane.interfaces.user import IUser

from Products.Membrane.types.UserMixin import Authentication, Properties, Groups

from Products.Membrane.config import PROJECTNAME, TOOLNAME

from Products.Archetypes.public import BaseSchema, Schema, BaseContent, StringField, StringWidget, registerType

SimpleSchema = BaseSchema + Schema((
    StringField('userName',
                languageIndependent = 1,
                widget = StringWidget(description = "Username for a person.")
               ),
    StringField('password',
                languageIndependent = 1,
                widget = StringWidget(description = "Password.")
               ),
    StringField('fullname',
                languageIndependent = 1,
                schemata='userinfo',
                user_property=True,
                widget = StringWidget(description = "Full name.")
               ),
    ))


class SimpleMember(Authentication, Properties, Groups, BaseContent):
    """A simple member archetype"""
    schema = SimpleSchema
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    #
    # For IAuthenticationPlugin implementation/Authentication mixin
    # - should define this as an interface
    #
    # getUserName is autogenerated
    #

    def _getPassword(self):
        return self.Schema()['password'].get(self)

    def getUserId(self):
        return self.UID()


    #
    # For IPropertiesPlugin implementation/Property mixin
    # - should probably use an interface
    #
    security.declarePrivate( 'getUserPropertySchematas' )
    def getUserPropertySchemata(self):
        return ['userinfo']

    #
    # For IGroupsPlugin implementation/Group mixin
    # - should probably use an interface
    #
    security.declarePrivate( 'getGroupRelationships' )
    def getGroupRelationships(self):
        return ['participatesInProject']


classImplements( SimpleMember
               , *tuple(implementedBy(SimpleMember)) +
                 (IUser,)) # Insert extra interfaces in empty tuple


registerType(SimpleMember, PROJECTNAME)
