# Copyright 2005 Plone Solutions
# info@plonesolutions.com

import copy
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from App.class_init import default__class_init__ as InitializeClass
from OFS.Cache import Cacheable
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import createViewName, classImplements
from Products.PluggableAuthService.interfaces.plugins \
    import IPropertiesPlugin

from Products.Membrane.config import TOOLNAME


manage_addMembranePropertyManagerForm = PageTemplateFile(
    '../www/MembranePropertyManagerForm', globals(), __name__='manage_addMembranePropertyManagerForm' )

def addMembranePropertyManager( dispatcher, id, title=None, REQUEST=None ):
    """ Add a MembranePropertyManager to a Pluggable Auth Service. """

    pmm = MembranePropertyManager(id, title)
    dispatcher._setObject(pmm.getId(), pmm)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'MembranePropertyManager+added.'
                            % dispatcher.absolute_url())

class MembranePropertyManager( BasePlugin, Cacheable ):

    """ PAS plugin for managing properties on contentish users in Plone.
    """

    meta_type = 'Membrane Property Manager'

    security = ClassSecurityInfo()

    def __init__(self, id, title=None):

        self._id = self.id = id
        self.title = title

    #
    #   IPropertiesPlugin implementation
    #
    security.declarePrivate( 'getPropertiesForUser' )
    def getPropertiesForUser(self, user, request=None ):
        properties = {}
        members = getToolByName(self, TOOLNAME).unrestrictedSearchResults( getUserId=user.getId()
                                                                         , object_implements='IPropertiesPlugin')
        # XXX THERE SHOULD PROBABLY BE ONLY ONE RESULT
        for m in members:
            member = m._unrestrictedGetObject()
            properties['uid'] = member.UID()
            if IPropertiesPlugin.isImplementedBy(member):
                properties.update(member.getPropertiesForUser(member, request))
        if properties.has_key('id'):
            del properties['id'] # When instantiating sheet(id, **props) is used - two ids is bad
        return properties


classImplements( MembranePropertyManager
               , IPropertiesPlugin
               )

InitializeClass( MembranePropertyManager )
