# Copyright 2005 Plone Solutions
# info@plonesolutions.com

import copy
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from App.class_init import default__class_init__ as InitializeClass
from OFS.Cache import Cacheable
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import createViewName, classImplements
from Products.PluggableAuthService.interfaces.plugins \
    import IRolesPlugin

from Products.Membrane.config import TOOLNAME


manage_addMembraneRoleManagerForm = PageTemplateFile(
    '../www/MembraneRoleManagerForm', globals(), __name__='manage_addMembraneRoleManager' )

def addMembraneRoleManager( dispatcher, id, title=None, REQUEST=None ):
    """ Add a MembraneRoleManager to a Pluggable Auth Service. """

    pmm = MembraneRoleManager(id, title)
    dispatcher._setObject(pmm.getId(), pmm)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'MembraneRoleManager+added.'
                            % dispatcher.absolute_url())

class MembraneRoleManager( BasePlugin, Cacheable ):

    """ PAS plugin for managing roles with Membrane.
    """

    meta_type = 'Membrane Role Manager'

    security = ClassSecurityInfo()

    def __init__(self, id, title=None):

        self._id = self.id = id
        self.title = title

    #
    #   IRolesPlugin implementation
    #
    security.declarePrivate( 'getRolesForPrincipal' )
    def getRolesForPrincipal(self, principal, request=None ):
        principals = getToolByName(self, TOOLNAME).unrestrictedSearchResults( getUserId=principal.getId()
                                                                            , object_implements='IRolesPlugin')
        roles = {}
        for p in principals:
            principal = p._unrestrictedGetObject()
            if IRolesPlugin.isImplementedBy(principal):
                roles.update(dict.fromkeys(principal.getRolesForPrincipal(principal)))
        # XXX WE HAVE TO GET ROLES FROM GROUPS AS WELL
#        groups = principal.getGroups()
#        if groups:
#            plugins = plugins = self.aq_parent._getOb('plugins')
#            rolemakers = plugins.listPlugins(IRolesPlugin)
#            for group in groups:
#                for rolemaker_id, rolemaker in rolemakers:
#                    # Have to get groups from IGroupsPlugin plugins, then getRolesForPrincipal
#                    grouproles = rolemaker.getRolesForPrincipal(group, request)
#                    if grouproles:
#                        roles.update(dict.fromkeys(grouproles))
        return tuple(roles.keys())


classImplements( MembraneRoleManager
               , IRolesPlugin
               )

InitializeClass( MembraneRoleManager )
