# Copyright 2005 Plone Solutions
# info@plonesolutions.com

import copy
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from App.class_init import default__class_init__ as InitializeClass
from OFS.Cache import Cacheable
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.utils import createViewName, classImplements
from Products.PluggableAuthService.interfaces.plugins \
    import IAuthenticationPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import IUserEnumerationPlugin

from Products.PlonePAS.interfaces.plugins import IUserIntrospection

from Products.Membrane.config import TOOLNAME

manage_addMembraneUserManagerForm = PageTemplateFile(
    '../www/MembraneUserManagerForm', globals(), __name__='manage_addMembraneUserManagerForm' )


def addMembraneUserManager( dispatcher, id, title=None, REQUEST=None ):
    """ Add a MembraneUserManager to a Pluggable Auth Service. """

    pmm = MembraneUserManager(id, title)
    dispatcher._setObject(pmm.getId(), pmm)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(
                                '%s/manage_workspace'
                                '?manage_tabs_message='
                                'MembraneUserManager+added.'
                            % dispatcher.absolute_url())


class MembraneUserManager( BasePlugin, Cacheable ):

    """ PAS plugin for managing contentish members in Plone.
    """

    meta_type = 'Membrane User Manager'

    security = ClassSecurityInfo()

    def __init__(self, id, title=None):

        self._id = self.id = id
        self.title = title

    #
    #   IAuthenticationPlugin implementation
    #
    security.declarePrivate( 'authenticateCredentials' )
    def authenticateCredentials( self, credentials ):

        """ See IAuthenticationPlugin.

        o We expect the credentials to be those returned by
          ILoginPasswordExtractionPlugin.
        """
        login = credentials.get( 'login' )
        password = credentials.get( 'password' )

        if login is None or password is None:
            return None

        # We can't depend on security when authenticating the user,
        # or we'll get stuck in loops
        members = getToolByName(self, TOOLNAME).unrestrictedSearchResults( getUserName=login
                                                                         , object_implements='IAuthenticationPlugin')

        for m in members:
            member = m._unrestrictedGetObject()
            # Delegate to member object
            if IAuthenticationPlugin.isImplementedBy(member):
                return member.authenticateCredentials(credentials)

        return None

    #
    #   IUserEnumerationPlugin implementation
    #
    security.declarePrivate( 'enumerateUsers' )
    def enumerateUsers( self
                      , id=None
                      , login=None
                      , exact_match=False
                      , sort_by=None
                      , max_results=None
                      , **kw
                      ):

        """ See IUserEnumerationPlugin.
        """
        user_info = []
        plugin_id = self.getId()
        view_name = createViewName('enumerateUsers', id or login)


        if isinstance( id, str ):
            id = [ id ]

        if isinstance( login, str ):
            login = [ login ]

        # Look in the cache first...
        keywords = copy.deepcopy(kw)
        keywords.update( { 'id' : id
                         , 'login' : login
                         , 'exact_match' : exact_match
                         , 'sort_by' : sort_by
                         , 'max_results' : max_results
                         }
                       )
        cached_info = self.ZCacheable_get( view_name=view_name
                                         , keywords=keywords
                                         , default=None
                                         )
        if cached_info is not None:
            return tuple(cached_info)

        membrane_tool = getToolByName(self, TOOLNAME)
        query = {}

        # Note: ZCTextIndex doesn't allow 'contains' searches AFAICT,
        #       so we use 'starts with'.
        if login:
            query['getUserName'] = exact_match and login or \
                                   ['%s*' % l for l in login]

        elif id:
            query['getUserId'] = exact_match and id or \
                                 ['%s*' % i for i in id]

        if sort_by is not None:
            if sort_by == 'login':
                query['sort_on'] = 'getUserName'
            if sort_by == 'id':
                query['sort_on'] = 'getUserId'

        query['object_implements'] = 'IUser'

        members = membrane_tool.unrestrictedSearchResults(**query)

        if max_results is not None:
            members = members[:max_results]

        for m in members:
            member = m._unrestrictedGetObject()
            info = { 'id': member.getUserId()
                     , 'login' : member.getUserName()
                     , 'pluginid': plugin_id
                     , 'editurl': '%s/base_edit' % member.absolute_url()
                     , 'uid': member.UID()
                     }
            user_info.append(info)
        
        # Put the computed value into the cache
        self.ZCacheable_set(user_info, view_name=view_name, keywords=keywords)

        return tuple( user_info )


    #
    #   IUserIntrospection implementation
    #
    security.declarePrivate( 'getUserIds' )
    def getUserIds(self):
        """
        Return a list of user ids
        """
        membrane_tool = getToolByName(self, TOOLNAME)
        users = membrane_tool.unrestrictedSearchResults(object_implements='IUser')
        return tuple([u.getUserId for u in users])

    security.declarePrivate( 'getUserNames' )
    def getUserNames(self):
        """
        Return a list of usernames
        """
        membrane_tool = getToolByName(self, TOOLNAME)
        users = membrane_tool.unrestrictedSearchResults(object_implements='IUser')
        return tuple([u.getUserName for u in users])

    security.declarePrivate( 'getUsers' )
    def getUsers(self):
        """
        Return a list of users
        """
        uf = self.acl_users
        return tuple([uf.getUserById(x) for x in self.getUserIds()])


classImplements( MembraneUserManager
               , IAuthenticationPlugin
               , IUserEnumerationPlugin
               , IUserIntrospection
               )


InitializeClass( MembraneUserManager )
