from AccessControl import ClassSecurityInfo

from Products.PluggableAuthService.utils import createViewName, classImplements, implementedBy

from Products.Membrane.interfaces.group import IGroup
from Products.Membrane.interfaces.user import IUser

from Products.Membrane.types.UserMixin import Authentication, Properties, SchemataProperties, Groups, SelectedGroups

from Products.Membrane.config import PROJECTNAME, TOOLNAME
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import BaseSchema, Schema, BaseFolder, BaseContent
from Products.Archetypes.public import ReferenceField, StringField
from Products.Archetypes.public import registerType
from Products.Archetypes.public import DisplayList

GROUP_RELATIONSHIP = 'participatesInProject'

group = BaseSchema + Schema((
    StringField('groupName',
                languageIndependent = 1,),
    ReferenceField(
        name='manager',
        relationship='managesProject',
        allowed_types=('Member','TestMember',),
        vocabulary='listUsers',
        languageIndependent = 1,),
    ReferenceField(
        name="members",
        relationship=GROUP_RELATIONSHIP,
        multiValued=1,
        languageIndependent = 1,
        allowed_types=('Member','TestMember',),
        vocabulary='listUsers',),
    ))


class TestGroup(BaseFolder):
    """A group archetype for testing"""
    schema = group

    security = ClassSecurityInfo()

    def getUserId(self):
        return self.getGroupName()

    def getGroupMembers(self):
        # All references and all subobjects that are members
        members = dict.fromkeys([m.getUserId() for m in self.getRefs(GROUP_RELATIONSHIP)])
        mt = getToolByName(self, TOOLNAME)
        for m in mt.unrestrictedSearchResults(object_implements='IUser', path='/'.join(self.getPhysicalPath())):
            members[m.getUserId] = 1
        return tuple(members.keys())

    def listUsers(self):
        """
        Return a DisplayList of users
        """
        catalog = getToolByName(self, TOOLNAME)

        results = catalog(object_implements='IUser')
        value = []
        for r in results:
            key = r.getUserName is not None and r.getUserName.strip() or r.getUserId
            value.append((key.lower(), (r.UID, key)))
        value.sort()
        value = [r for throwaway, r in value]
        value.insert(0, ('', '<no reference>'))
        return DisplayList(value)


classImplements( TestGroup
               , *tuple(implementedBy(BaseFolder)) +
                 (IGroup,))


registerType(TestGroup, PROJECTNAME)


user = BaseSchema + Schema((
    StringField('userName',
                languageIndependent = 1,),
    StringField('password',
                languageIndependent = 1,),
    StringField('fullname',
                languageIndependent = 1,
                schemata='userinfo',
                user_property=True,),
    ))


class BaseMember:
    schema = user
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    def _getPassword(self):
        return self.Schema()['password'].get(self)

    def getUserId(self):
        return self.UID()


class TestMember(BaseMember, Authentication, Properties, Groups, BaseContent):
    """A member archetype for testing"""
    pass


classImplements( TestMember
               , *tuple(implementedBy(TestMember)) +
                 (IUser,)) # Insert extra interfaces in empty tuple


registerType(TestMember, PROJECTNAME)


class AlternativeTestMember(BaseMember, Authentication, SchemataProperties, SelectedGroups, BaseContent):
    """A member archetype for testing"""

    security = ClassSecurityInfo()

    # getUserName is autogenerated

    # For IPropertiesPlugin implementation/Property mixin
    security.declarePrivate( 'getUserPropertySchematas' )
    def getUserPropertySchemata(self):
        return ['userinfo']

    # For IGroupsPlugin implementation/Group mixin
    security.declarePrivate( 'getGroupRelationships' )
    def getGroupRelationships(self):
        return [GROUP_RELATIONSHIP]


classImplements( AlternativeTestMember
               , *tuple(implementedBy(AlternativeTestMember)) +
                 (IUser,)) # Insert extra interfaces in empty tuple


registerType(AlternativeTestMember, PROJECTNAME)
