#
# MembraneTestCase Membrane
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest

from Testing import ZopeTestCase
from Products.Membrane.tests import MembraneTestCase
from Products.CMFPlone.utils import _createObjectByType

from Products.PluggableAuthService.tests.conformance \
    import IGroupsPlugin_conformance
from Products.PluggableAuthService.tests.conformance \
    import IGroupEnumerationPlugin_conformance

from Products.Membrane.tests.utils import sortTuple


class MembraneGroupManagerTestBase:

    def _getTargetClass( self ):

        from Products.Membrane.plugins.MembraneGroupManager \
            import MembraneGroupManager

        return MembraneGroupManager

    def _makeOne( self, id='test', *args, **kw ):

        return self._getTargetClass()( id=id, *args, **kw )

    def createGroupAndUsers(self):
        self.addGroup(self.portal)
        self.addUser(self.group)

        self.member2 = _createObjectByType('TestMember', self.portal, 'testmember')
        self.member2.setUserName('testmember')
        self.member2.setPassword('testpassword')
        self.member2.setFullname('Member 2')
        self.member2.reindexObject()


class TestMembraneGroupManagerBasics( unittest.TestCase
                                    , MembraneGroupManagerTestBase
                                    , IGroupsPlugin_conformance
                                    , IGroupEnumerationPlugin_conformance
                                    ):
    # Run the conformance tests
    pass


class TestMembraneGroupManager( MembraneTestCase.MembraneTestCase
                              , MembraneGroupManagerTestBase):

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.createGroupAndUsers()

    def testGroupMembership(self):
        group = self.portal.testgroup
        member = group.testuser # We need acquisition to be correct
        member2 = self.member2
        self.failUnlessEqual(group.getGroupMembers(), (member.getUserId(),))
        self.failUnlessEqual(member.getGroupsForPrincipal(member), (group.getId(),))
        self.group.setMembers([member2.UID()])
        self.failUnlessEqual(sortTuple(group.getGroupMembers()), sortTuple([member2.getUserId(), member.getUserId()]))
        self.failUnlessEqual(member2.getGroupsForPrincipal(member2), (group.getId(),))


class TestMembraneGroupManagerSelectedGroups( MembraneTestCase.MembraneTestCase
                                            , MembraneGroupManagerTestBase):

    def addUser(self, obj=None):
        if obj is None:
            obj = self.portal
        self.member = _createObjectByType('AlternativeTestMember', obj, 'testuser')
        self.member.setUserName('testuser')
        self.member.setPassword('testpassword')
        self.member.setFullname('full name')
        self.member.reindexObject()

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.createGroupAndUsers()

    def testGroupMembership(self):
        group = self.portal.testgroup
        member = group.testuser # We need acquisition to be correct
        member2 = self.member2
        self.failUnlessEqual(group.getGroupMembers(), (member.getUserId(),))
        self.failUnlessEqual(member.getGroupsForPrincipal(member), (group.getId(),))
        self.group.setMembers([member2.UID()])
        self.failUnlessEqual(sortTuple(group.getGroupMembers()), sortTuple([member2.getUserId(), member.getUserId()]))
        self.failUnlessEqual(member2.getGroupsForPrincipal(member2), (group.getId(),))


class TestMembraneGroupManagerEnumeration( MembraneTestCase.MembraneTestCase
                                         , MembraneGroupManagerTestBase):

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.createGroupAndUsers()

    def testEnumerateGroupsNoArgs(self):
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups()), 1)

    def testEnumerateGroupsByTitleNonexisting(self):
        self.failUnlessEqual(self.portal.pmm.enumerateGroups(title='nonexisting'), ())
        self.failUnlessEqual(self.portal.pmm.enumerateGroups(title='nonexisting', exact_match=True), ())

    def testEnumerateGroupsByTitle(self):
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title(), exact_match=True)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title()[:len(self.group.Title())-1], exact_match=False)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title(), exact_match=True, sort_on='title')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title(), exact_match=True, sort_on='id')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title(), exact_match=True, max_results=1)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(title=self.group.Title(), exact_match=True, max_results=0)), 0)

    def testEnumerateGroupsByIdNonexisting(self):
        self.failUnlessEqual(self.portal.pmm.enumerateGroups(id='nonexisting'), ())
        self.failUnlessEqual(self.portal.pmm.enumerateGroups(id='nonexisting', exact_match=True), ())

    def testEnumerateGroupsById(self):
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName(), exact_match=True)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName()[:len(self.group.getGroupName())-1], exact_match=False)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName(), exact_match=True, sort_on='title')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName(), exact_match=True, sort_on='id')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName(), exact_match=True, max_results=1)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateGroups(id=self.group.getGroupName(), exact_match=True, max_results=0)), 0)


class TestMembraneGroupIntrospection( MembraneTestCase.MembraneTestCase
                                    , MembraneGroupManagerTestBase):

    # Test IGroupIntrospection

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')

    def testGetGroupIdsNoGroups(self):
        self.failIf(self.portal.pmm.getGroupIds())

    def testGetGroupIds(self):
        self.addGroup(self.portal)
        groupids = self.portal.pmm.getGroupIds()
        self.failUnlessEqual(groupids, (self.group.getUserId(),))

        group2 = _createObjectByType('TestGroup', self.portal, 'testgroup2')
        group2.setTitle('Test group 2')
        group2.setGroupName('testgroup2')
        group2.reindexObject()
        groupids = self.portal.pmm.getGroupIds()
        self.failUnlessEqual(sortTuple(groupids), sortTuple((self.group.getUserId(),group2.getUserId())))

    def testGroupMembersNoMembers(self):
        self.addGroup(self.portal)
        self.failIf(self.portal.pmm.getGroupMembers(self.group.getUserId()))

    def testGroupMembers(self):
        self.createGroupAndUsers()
        self.failUnlessEqual(self.portal.pmm.getGroupMembers(self.group.getUserId()), (self.member.getUserId(),))

    def testGetGroupByIdNoGroup(self):
        g = self.portal.pmm.getGroupById('nonexisting')
        self.failIf(g)

    def testGetGroupById(self):
        from Products.PlonePAS.plugins.group import PloneGroup
        self.createGroupAndUsers()
        group = self.portal.pmm.getGroupById(self.group.getUserId())
        self.failUnless(group)
        self.failUnless(isinstance(group, PloneGroup))

    def testGetGroupsNoGroups(self):
        self.failIf(self.portal.pmm.getGroups())

    def testGetGroups(self):
        self.createGroupAndUsers()
        groups = self.portal.pmm.getGroups()
        self.failUnless(groups)
        self.failUnlessEqual(len(groups), 1)
        self.failUnlessEqual(groups[0].getUserId(), self.group.getUserId())


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMembraneGroupManagerBasics))
    suite.addTest(makeSuite(TestMembraneGroupManager))
    suite.addTest(makeSuite(TestMembraneGroupManagerSelectedGroups))
    suite.addTest(makeSuite(TestMembraneGroupManagerEnumeration))
    suite.addTest(makeSuite(TestMembraneGroupIntrospection))
    return suite

if __name__ == '__main__':
    framework()
