#
# MembraneTestCase Membrane
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest

from Testing import ZopeTestCase
from Products.Membrane.tests import MembraneTestCase

from Products.CMFPlone.utils import _createObjectByType

from Products.PluggableAuthService.tests.conformance \
    import IAuthenticationPlugin_conformance
from Products.PluggableAuthService.tests.conformance \
    import IUserEnumerationPlugin_conformance
from Products.PluggableAuthService.tests.conformance \
    import IGroupsPlugin_conformance
from Products.PluggableAuthService.tests.conformance \
    import IGroupEnumerationPlugin_conformance

class MembranePropertyManagerTestBase:

    def _getTargetClass( self ):

        from Products.Membrane.plugins.MembranePropertyManager \
            import MembranePropertyManager

        return MembranePropertyManager

    def _makeOne( self, id='test', *args, **kw ):

        return self._getTargetClass()( id=id, *args, **kw )


class User:
    def __init__(self, member):
        self.member = member
    def getId(self):
        return self.member.getUserId()


class TestMembranePropertyManager( MembraneTestCase.MembraneTestCase
                                 , MembranePropertyManagerTestBase):

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.addUser()

    def testGetPropertiesForUserOnUser(self):
        properties = self.member.getPropertiesForUser(None)

        self.failUnless(properties.has_key('fullname'))

    def testGetPropertiesForUserFromPropertyManager(self):
        properties = self.portal.pmm.getPropertiesForUser(User(self.member))
        self.failIf(properties.has_key('id'))
        self.failUnless(properties.has_key('fullname'))

    def testGetPropertiesForUser(self):
        user = self.portal.acl_users.getUserById(self.member.getUserId())
        sheets = user.getOrderedPropertySheets()
        self.failUnless([x.getProperty('fullname') for x in sheets if x.getProperty('fullname') == 'full name'])
        member = self.portal.portal_membership.getMemberById(self.member.getUserId())
        self.failUnlessEqual(member.getProperty('fullname'), 'full name')


class TestMembraneSchemataPropertyManager( MembraneTestCase.MembraneTestCase
                                         , MembranePropertyManagerTestBase):

    def addUser(self, obj=None):
        if obj is None:
            obj = self.portal
        self.member = _createObjectByType('AlternativeTestMember', obj, 'testuser')
        self.member.setUserName('testuser')
        self.member.setPassword('testpassword')
        self.member.setFullname('full name')
        self.member.reindexObject()

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.addUser()

    def testGetPropertiesForUserOnUser(self):
        properties = self.member.getPropertiesForUser(None)

        self.failUnless(properties.has_key('fullname'))

    def testGetPropertiesForUserFromPropertyManager(self):
        properties = self.portal.pmm.getPropertiesForUser(User(self.member))
        self.failIf(properties.has_key('id'))
        self.failUnless(properties.has_key('fullname'))

    def testGetPropertiesForUser(self):
        user = self.portal.acl_users.getUserById(self.member.getUserId())
        sheets = user.getOrderedPropertySheets()
        self.failUnless([x.getProperty('fullname') for x in sheets if x.getProperty('fullname') == 'full name'])
        member = self.portal.portal_membership.getMemberById(self.member.getUserId())
        self.failUnlessEqual(member.getProperty('fullname'), 'full name')


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMembranePropertyManager))
    suite.addTest(makeSuite(TestMembraneSchemataPropertyManager))
    return suite

if __name__ == '__main__':
    framework()
