#
# MembraneTestCase Membrane
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

import unittest

from Testing import ZopeTestCase
from Products.Membrane.tests import MembraneTestCase
from Products.CMFPlone.utils import _createObjectByType

from Products.PluggableAuthService.tests.conformance \
    import IAuthenticationPlugin_conformance
from Products.PluggableAuthService.tests.conformance \
    import IUserEnumerationPlugin_conformance

from Products.Membrane.tests.utils import sortTuple

class MembraneUserManagerTestBase:

    def _getTargetClass( self ):

        from Products.Membrane.plugins.MembraneUserManager \
            import MembraneUserManager

        return MembraneUserManager

    def _makeOne( self, id='test', *args, **kw ):

        return self._getTargetClass()( id=id, *args, **kw )
        

class TestMembraneUserManagerBasics( unittest.TestCase
                             , MembraneUserManagerTestBase
                             , IAuthenticationPlugin_conformance
                             , IUserEnumerationPlugin_conformance
                             ):
    # Run the conformance tests
    pass


class TestMembraneUserManagerEnumeration( MembraneTestCase.MembraneTestCase
                                        , MembraneUserManagerTestBase):

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.addUser()

    def testEnumerateUsersNoArgs(self):
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers()), 1)        

    def testEnumerateUsersByLoginNonexisting(self):
        self.failUnlessEqual(self.portal.pmm.enumerateUsers(login='nonexisting'), ())
        self.failUnlessEqual(self.portal.pmm.enumerateUsers(login='nonexisting', exact_match=True), ())

    def testEnumerateUsersByLogin(self):
        username = self.member.getUserName()
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username[:len(username)-1], exact_match=False)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True, sort_on='login')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True, sort_on='id')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True, max_results=1)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(login=username, exact_match=True, max_results=0)), 0)

    def testEnumerateUsersByUserIdNonexisting(self):
        self.failUnlessEqual(self.portal.pmm.enumerateUsers(id='nonexisting'), ())
        self.failUnlessEqual(self.portal.pmm.enumerateUsers(id='nonexisting', exact_match=True), ())

    def testEnumerateUsersByUserId(self):
        userid = self.member.getUserId()
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid, exact_match=True)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid[:len(userid)-1], exact_match=False)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid, exact_match=True, sort_on='login')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid, exact_match=True, sort_on='id')), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid, exact_match=True, max_results=1)), 1)
        self.failUnlessEqual(len(self.portal.pmm.enumerateUsers(id=userid, exact_match=True, max_results=0)), 0)


class TestMembraneUserManagerAuthentication( MembraneTestCase.MembraneTestCase
                                           , MembraneUserManagerTestBase):
    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.addUser()

    def testAuthenticateOnMember(self):
        credentials = {'login':'norealuser', 'password':'norealpassword'}
        self.failUnlessEqual(self.member.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'wrongpassword'}
        self.failUnlessEqual(self.member.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'testpassword'}
        self.failUnlessEqual(self.member.authenticateCredentials(credentials), (self.member.getUserId(), self.member.getUserName()))

    def testAuthenticate(self):
        credentials = {'login':'norealuser', 'password':'norealpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'wrongpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'testpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), (self.member.getUserId(), self.member.getUserName()))

    def testLogin(self):
        self.login(self.member.getUserId())


class TestMembraneUserManagerAuthenticationPermissions( MembraneTestCase.MembraneTestCase
                                                      , MembraneUserManagerTestBase):
    """Check if everything works when the user object is private"""

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')
        self.addUser()
        self.portal.portal_workflow.doActionFor(self.member, 'hide')

    def testAuthenticate(self):
        self.logout()
        credentials = {'login':'norealuser', 'password':'norealpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'wrongpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), None)
        credentials = {'login':'testuser', 'password':'testpassword'}
        self.failUnlessEqual(self.portal.pmm.authenticateCredentials(credentials), (self.member.getUserId(), self.member.getUserName()))

    def testLogin(self):
        self.logout()
        self.login(self.member.getUserId())


class TestMembraneUserManagerIntrospection( MembraneTestCase.MembraneTestCase
                                          , MembraneUserManagerTestBase):

    def createSecondUser(self):
        self.member2 = _createObjectByType('TestMember', self.portal, 'testuser2')
        self.member2.setUserName('testuser2')
        self.member2.setPassword('testpassword2')
        self.member2.setFullname('full name 2')
        self.member2.reindexObject()

    def afterSetUp(self):
        self.portal.pmm = self._makeOne('pmm')

    def testGetUserIdsNoUsers(self):
        self.failIf(self.portal.pmm.getUserIds())

    def testGetUserIdsOneUser(self):
        self.addUser()
        self.failUnlessEqual(self.portal.pmm.getUserIds(), (self.member.getUserId(),))

    def testGetUserIds(self):
        self.addUser()
        self.createSecondUser()
        userids = sortTuple(self.portal.pmm.getUserIds())
        self.failUnlessEqual(userids, sortTuple((self.member.getUserId(), self.member2.getUserId())))

    def getUserNamesNoUsers(self):
        self.failIf(self.portal.pmm.getUserNames())

    def getUserNamesOneUser(self):
        self.addUser()
        self.failUnlessEqual(self.portal.pmm.getUserNames(), (self.member.getUserName(),))

    def getUserNames(self):
        self.addUser()
        self.createSecondUser()
        usernames = sortTuple(self.portal.pmm.getUserIds())
        self.failUnlessEqual(userids, sortTuple((self.member.getUserName(), self.member2.getUserName())))

    def testGetUsersNoUsers(self):
        self.failIf(self.portal.pmm.getUsers())

    def testGetUsersOneUser(self):
        self.addUser()
        users = self.portal.pmm.getUsers()
        self.failUnlessEqual([x.getId() for x in users], [self.member.getUserId()])

    def testGetUsers(self):
        self.addUser()
        self.createSecondUser()
        usernames = sortTuple(self.portal.pmm.getUserIds())
        self.failUnlessEqual(usernames, sortTuple((self.member.getUserId(), self.member2.getUserId())))


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMembraneUserManagerBasics))
    suite.addTest(makeSuite(TestMembraneUserManagerEnumeration))
    suite.addTest(makeSuite(TestMembraneUserManagerAuthentication))
    suite.addTest(makeSuite(TestMembraneUserManagerAuthenticationPermissions))
    suite.addTest(makeSuite(TestMembraneUserManagerIntrospection))
    return suite

if __name__ == '__main__':
    framework()
