from AccessControl import ClassSecurityInfo, AuthEncoding
from Acquisition import aq_chain, aq_inner
from App.class_init import default__class_init__ as InitializeClass

from Products.CMFCore.utils import getToolByName

from Products.PluggableAuthService.utils import createViewName, classImplements, implementedBy
from Products.PluggableAuthService.interfaces.plugins \
    import IAuthenticationPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import IPropertiesPlugin
from Products.PluggableAuthService.interfaces.plugins \
    import IGroupsPlugin

from Products.Membrane.interfaces.group import IGroup
from Products.Membrane.interfaces.user import IUser

from Products.Membrane.config import PROJECTNAME, TOOLNAME
from Products.Archetypes.public import BaseSchema, Schema, BaseContent, StringField, StringWidget, registerType


class UserMarker:
    pass


classImplements( UserMarker
               , IUser
               )


InitializeClass( UserMarker )


class Authentication:
    """Authentication mixin. 
    Assumes there is a getUserId and a _getPassword method available"""

    security = ClassSecurityInfo()

    security.declarePrivate( 'getUserId' )
    def getUserId(self):
        # UID can work, so can id or LDAP dn
        return self.getId()

    security.declarePrivate( '_getPassword' )
    def _getPassword(self):
        return None

    #
    #   IAuthenticationPlugin implementation
    #
    security.declarePrivate( 'authenticateCredentials' )
    def authenticateCredentials( self, credentials ):
        login = credentials.get( 'login' )
        password = credentials.get( 'password' )

        if login is None or password is None:
            return None

        userid = self.getUserId()

        reference = self._getPassword()

        if reference is None:
            return None

        if AuthEncoding.is_encrypted( reference ):
            if AuthEncoding.pw_validate( reference, password ):
                return userid, login

        elif reference == password:
            return userid, login


classImplements( Authentication
               , IAuthenticationPlugin
               )


InitializeClass( Authentication )


class Properties:
    """This property mixin gets properties marked as user_property"""

    security = ClassSecurityInfo()

    #
    #   IPropertiesPlugin implementation
    #
    security.declarePrivate( 'getPropertiesForUser' )
    def getPropertiesForUser(self, user, request=None ):
        illegal_ids = ['id']
        properties = {}
        schema = self.Schema()
        for field in schema.fields():
            if hasattr(field, 'user_property') and field.getName() not in illegal_ids:
                value = field.get(self)
                properties[field.getName()] = value is not None and value or ''
        return properties


classImplements( Properties
               , IPropertiesPlugin
               )


InitializeClass( Properties )


class SchemataProperties:
    """This property mixin gets properties from the selected schematas"""

    security = ClassSecurityInfo()

    security.declarePrivate( 'getUserPropertySchemata' )
    def getUserPropertySchemata(self):
        return ['default']

    #
    #   IPropertiesPlugin implementation
    #
    security.declarePrivate( 'getPropertiesForUser' )
    def getPropertiesForUser(self, user, request=None ):
        illegal_ids = ['id']
        properties = {}
        schemata = self.Schemata()
        for ups in self.getUserPropertySchemata():
            schema = schemata.get(ups, None)
            if schema is not None:
                for field in schema.fields():
                    if field.getName() not in illegal_ids:
                        value = field.get(self)
                        properties[field.getName()] = value is not None and value or ''
        return properties


classImplements( SchemataProperties
               , IPropertiesPlugin
               )


InitializeClass( SchemataProperties )


class Groups:
    """This group mixin gets groups from acquisition and backrefs"""

    security = ClassSecurityInfo()

    #
    #   IGroupsPlugin implementation
    #
    security.declarePrivate( 'getGroupsForPrincipal' )
    def getGroupsForPrincipal(self, principal, request=None ):
        assert(principal == self)
        groups = {}
        # Get all BRefs that implement IGroup - slightly expensive
        for obj in self.getBRefs():
            if IGroup.isImplementedBy(obj):
                groups[obj.getUserId()] = 1
        for parent in aq_chain(aq_inner(self)):
            if IGroup.isImplementedBy(parent):
                groups[parent.getUserId()] = 1
        return tuple(groups.keys())


classImplements( Groups
               , IGroupsPlugin
               )


InitializeClass( Groups )


class SelectedGroups:
    """This group mixin gets groups from acquisition and backrefs"""

    security = ClassSecurityInfo()

    security.declarePrivate( 'getGroupRelationships' )
    def getGroupRelationships(self):
        return []

    #
    #   IGroupsPlugin implementation
    #
    security.declarePrivate( 'getGroupsForPrincipal' )
    def getGroupsForPrincipal(self, principal, request=None ):
        assert(principal == self)
        groups = {}
        for relationship in self.getGroupRelationships():
            groups.update(dict.fromkeys([g.getUserId() for g in self.getBRefs(relationship)]))
        for parent in aq_chain(aq_inner(self)):
            if IGroup.isImplementedBy(parent):
                groups[parent.getUserId()] = 1
        return tuple(groups.keys())


classImplements( SelectedGroups
               , IGroupsPlugin
               )


InitializeClass( SelectedGroups )
