from Products.Archetypes.public import *
from Products.Membrane.permissions import VIEW_PUBLIC_PERMISSION, \
     EDIT_ID_PERMISSION, VIEW_OTHER_PERMISSION, EDIT_PROPERTIES_PERMISSION, \
     VIEW_SECURITY_PERMISSION, EDIT_PASSWORD_PERMISSION, \
     EDIT_SECURITY_PERMISSION, MAIL_PASSWORD_PERMISSION, ADD_MEMBER_PERMISSION, \
     VIEW_PERMISSION, REGISTER_PERMISSION


# Use schema from CMFMember
id_schema = Schema((
    StringField('id',
                required=1,
                accessor='getId',
                mutator='setId',
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_ID_PERMISSION,
                default=None,
#                index=('member_catalog/ZCTextIndex,lexicon_id=member_lexicon,index_type=Cosine Measure|TextIndex:brains',
#                       'FieldIndex:brains'),
                widget=IdWidget(
                    label='User name',
                    label_msgid='label_user_name',
                    size=10,
                    maxlength=25,
                    description="Enter a user name, usually something like "
                        "'jsmith'. No spaces or special characters. User "
                        "names and passwords are case sensitive, make sure "
                        "the capslock key is not enabled. This is the name "
                        "used to log in.",
                    description_msgid='help_user_name_creation_casesensitive',
                    i18n_domain='cmfmember',
                    display_autogenerated=0,
#                    macro='memid'
                    ),
                regfield=1,  ### this field is part of the registration form
                ),


    StringField('title',
                required=0,
                searchable=1,
                default='',
                accessor='Title',
                mutator='setTitle',
                widget=StringWidget(
                    label_msgid="label_title",
                    i18n_domain="plone",
                    visible = { 'edit' :'invisible', 'view' : 'invisible' }
                    ),
                ),
))


contact_schema = Schema((
    StringField('fullname',
                default='',
                accessor='getFullName',
                mutator='setFullName',
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
#                index=('member_catalog/ZCTextIndex,lexicon_id=member_lexicon,index_type=Cosine Measure|TextIndex:brains',
#                       'ZCTextIndex|TextIndex:brains'),
                widget=StringWidget(
                    label='Full name',
                    label_msgid='label_full_name',
                    description="Enter full name, eg. John Smith.",
                    description_msgid='help_full_name_creation',
                    i18n_domain='cmfmember',
                    ),
                regfield=1,
                ),

    StringField('email',
                required=1,
                accessor='getEmail',
                mutator='setEmail',
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                validators=('isEmail',),
#                index=('member_catalog/ZCTextIndex,lexicon_id=member_lexicon,index_type=Cosine Measure|TextIndex:brains',
#                       'ZCTextIndex|TextIndex:brains'),
                widget=StringWidget(
                    label='E-mail',
                    label_msgid='label_email',
                    description="Enter an email address. This is necessary "
                        "in case the password is lost. We respect your privacy "
                        "and will not give the address away to any third "
                        "parties or expose it anywhere.",
                    description_msgid='help_email_creation',
                    i18n_domain='cmfmember',
                    ),
                regfield=1,
                ),
))


plone_schema = Schema((
    StringField('wysiwyg_editor',
                mode='rw',
                read_permission=VIEW_OTHER_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                vocabulary='editors',
                enforceVocabulary=1,
                widget=SelectionWidget(
                    format='flex',
                    label='Content editor',
                    label_msgid='label_content_editor',
                    description="Select the editor that you would like to use "
                        "for editing content more easily. Content editors "
                        "often have specific browser requirements.",
                    description_msgid='help_content_editor',
                    i18n_domain='cmfmember',
                    ),
                ),


    BooleanField('visible_ids',
                default=1,
                mode='rw',
                read_permission=VIEW_OTHER_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                widget=BooleanWidget(
                    label='Display names',
                    label_msgid='label_edit_short_names',
                    description="Determines if Short Names (also known as "
                        "IDs) are changable when editing items. If Short "
                        "Names are not displayed, they will be generated "
                        "automatically.",
                    description_msgid='help_display_names',
                    i18n_domain='cmfmember',
                    ),
                ),

    StringField('portal_skin',
                mode='rw',
                default=None,
                read_permission=VIEW_OTHER_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                accessor='getPortalSkin',
                vocabulary='available_skins',
                widget=SelectionWidget(
                    format='flex',
                    label='Look',
                    label_msgid='label_look',
                    description="Appearance of the site.",
                    description_msgid='help_look',
                    i18n_domain='cmfmember',
                    condition="portal/portal_skins/allow_any",
                    ),
                ),

    ImageField('portrait',
               mode='rw',
               accessor='getPortrait',
               mutator='setPortrait',
               max_size=(150,150),
               read_permission=VIEW_PUBLIC_PERMISSION,
               write_permission=EDIT_PROPERTIES_PERMISSION,
               required=0,
               widget=ImageWidget(
                   label='Portrait',
                   label_msgid='label_portrait',
                   description="To add or change the portrait: click the "
                       "\"Browse\" button; select a picture of yourself. "
                       "Recommended image size is 75 pixels wide by 100 "
                       "pixels tall.",
                   description_msgid='help_portrait',
                   i18n_domain='cmfmember',
                   ),
               ),

))


security_schema = Schema((
    StringField('password',
                mutator='_setPassword',
                accessor='getPassword',
                mode='w',
                write_permission=EDIT_PASSWORD_PERMISSION,
                widget=PasswordWidget(
                    label='Password',
                    label_msgid='label_password',
                    description="Minimum 5 characters",
                    visible = {'view' : 'invisible' },
                    description_msgid='help_password_creation',
                    i18n_domain='cmfmember',
                    condition="object/showPasswordOnRegistration",
                    ),
                regfield=1,
                ),

    StringField('confirm_password',
                mutator='_setConfirmPassword',
                accessor='_getConfirmPassword',
                mode='w',
                read_permission=VIEW_SECURITY_PERMISSION,
                write_permission=EDIT_PASSWORD_PERMISSION,
                widget=PasswordWidget(
                    label='Confirm password',
                    label_msgid='label_confirm_password',
                    description="Re-enter the password. Make sure the "
                        "passwords are identical.",
                    description_msgid='help_confirm_password',
                    i18n_domain='cmfmember',
                    condition="object/showPasswordOnRegistration",
                    ),
                regfield=1,
                ),

   BooleanField('mail_me',
                default=0,
                mode='w',
                searchable = 0,
                write_permission=EDIT_PASSWORD_PERMISSION,
                widget=BooleanWidget(
                    label='Send a mail with the password',
                    label_msgid='label_mail_password',
                    description='',
                    i18n_domain='cmfmember',
                    condition="object/showPasswordOnRegistration",
                    ),
                regfield=1,
                ),

    LinesField('roles',
               default=('Member',),
               mutator='setRoles',
               accessor='getFilteredRoles',
               mode='rw',
               read_permission=VIEW_SECURITY_PERMISSION,
               write_permission=EDIT_SECURITY_PERMISSION,
               vocabulary='valid_roles',
#               enforceVocabulary=1, # don't enforce vocabulary because getRoles() adds some extra roles
               multiValued=1,
#               index='member_catalog/KeywordIndex:brains',
               widget=MultiSelectionWidget(
                   label='Roles',
                   label_msgid='label_roles',
                   description="Select the security roles for this user.",
                   description_msgid='help_select_member_role',
                   i18n_domain='cmfmember',
                   )
               ),

    LinesField('groups',
               default=(),
               mutator='setGroups',
               accessor='getGroups',
               edit_accessor='getGroups',
               mode='rw',
               read_permission=VIEW_SECURITY_PERMISSION,
               write_permission=EDIT_SECURITY_PERMISSION,
               vocabulary='valid_groups',
               enforceVocabulary=1,
               multiValued=1,
#               index='member_catalog/KeywordIndex:brains',
               widget=MultiSelectionWidget(
                   label='Groups',
                   label_msgid='label_groups',
                   description="Indicate the groups to which this member "
                       "belongs.",
                   description_msgid='help_select_member_groups',
                   i18n_domain='cmfmember',
                   )
               ),

    LinesField('domains',
               default=(),
               mutator='setDomains',
               accessor='getDomains',
               mode='rw',
               read_permission=VIEW_SECURITY_PERMISSION,
               write_permission=EDIT_SECURITY_PERMISSION,
               multivalued=1,
               widget=LinesWidget(
                   label='Domains',
                   label_msgid='label_member_domains',
                   description="If you would like to restrict this user to "
                       "logging in only from certain domains, enter those "
                       "domains here.",
                   description_msgid='help_member_domains',
                   i18n_domain='cmfmember',
                   )
               ),

     ComputedField('review_state',
                   mode='r',
                   read_permission=VIEW_SECURITY_PERMISSION,
                   expression="context.portal_workflow.getInfoFor(context, 'review_state')",
#                   index=('member_catalog/FieldIndex:brains',),
                   widget=ComputedWidget(
                       label="Status",
                       label_msgid='label_review_state',
                       modes=('view',),
                       visible={'edit':'invisible',
                                'view':'visible'},
                       i18n_domain='cmfmember',
                       ),
                   ),
))


login_info_schema = Schema((
    DateTimeField('login_time',
                  default='2000/01/01',  # for Plone 1.0.1 compatibility
                  mode='r',
                  read_permission=VIEW_OTHER_PERMISSION,
                  write_permission=EDIT_PROPERTIES_PERMISSION,
                  widget=StringWidget(
                      label="Login time",
                      modes=('view',),
                      visible={'edit':'invisible',
                               'view':'visible'},
                      ),
                  ),

    DateTimeField('last_login_time',
                  default='2000/01/01',  # for Plone 1.0.1 compatibility
                  mode='r',
                  accessor='getLastLoginTime',
                  mutator='setLastLoginTime',
                  read_permission=VIEW_OTHER_PERMISSION,
                  write_permission=EDIT_PROPERTIES_PERMISSION,
#                  index='member_catalog/DateIndex:brains',
                  widget=StringWidget(
                      label="Last login time",
                      modes=('view',),
                      visible={'edit':'invisible',
                               'view':'visible'},
                      ),
                  ),

    ComputedField('listed',
                  mode='r',
                  read_permission=VIEW_OTHER_PERMISSION,
                  expression="context.isListed()",
                  widget=ComputedWidget(
                      label="Listed",
                      label_msgid='label_listed_status',
                      modes=('view',),
                      visible={'edit':'invisible',
                               'view':'visible'},
                      i18n_domain='cmfmember',
                      ),
                  ),
))

content_schema = id_schema + contact_schema + plone_schema + security_schema \
                 + login_info_schema


